unit boxplotu;

{ ***************************************************************************

  How to create Boxplots with TChart - a framework for educational purposes
  (c) 2001 Michael Schumann, Marburg - micha@itc-ms.de
  This Example was written in Delphi 5 Enterprise but should also work with
  other Versions since no version specific stuff is used.

  Why I wrote this:
  For a statistical project that makes heave use of TChart (I do like it!) I
  needed a way to present longitudinal mass data together with individual
  values i a kind of benchmark. Since TChart already was integrated I put
  this code on top to create the boxplots.

  Usage:
  This code is released as Freeware.
  Use the code at your own risk. If you mention my name in a product you use
  the code in, I will not complain, but you do not have to do so!
  In no circumstances I will be held liable for any damages or loss of income
  resulting from the use of that code. You have the source - read it, under-
  stand it, expand and modify it and integrate it. If you make enhancements,
  please send me a copy!

  Michael Schumann, August 2001 on holiday

  *************************************************************************** }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, TeEngine, Series, ExtCtrls, TeeProcs, Chart, ITCUtils, ITCStrDate;

type
  TForm1 = class(TForm)
    Chart1: TChart;
    Series1: TLineSeries;
    Button3: TButton;
    procedure Chart1GetAxisLabel(Sender: TChartAxis; Series: TChartSeries;
      ValueIndex: Integer; var LabelText: String);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
  private
    { Private-Deklarationen }
    boxPlotActive:boolean;
    byy0,bDX,bDY,bX0,bY0: real;
    boxCount: integer;
    function PixY(y:real):integer; // calculate canvas pos from y-value
  public
    { Public-Deklarationen }
    procedure prepareBoxPlot(Ymin,YMax:real;Count:integer); // sets ranges and labels
    procedure DrawBox(x:integer;y0,y1,y2,y3,y4,ym:real);
    procedure DrawPoint(x:integer;y:real);
    procedure freeBoxPlot; // remove Box-Plot and settings
  end;

var
  Form1: TForm1;

const
  Dist= 12;  // Distance from x-Value to show Boxplot and individual values
             // if not used set to 0
  bWB = 28;  // width of box
  bWA = 10;   // width of antenna
  bWM = 5;   // Size of Mean-Cross

implementation

{$R *.DFM}

procedure TForm1.Chart1GetAxisLabel(Sender: TChartAxis;
  Series: TChartSeries; ValueIndex: Integer; var LabelText: String);
var x:integer;
begin
  // set customized labels for x-axis in Boxplot-Mode
  if boxPlotActive and (Sender=Chart1.BottomAxis) then begin
    // Customized X-Labels e.g. Quarters except first and last
    // ValueIndex wont work since we have no series values!
    x:=StrToInt(LabelText);
    if (x>0) and (x<=BoxCount) then
      LabelText:='Quarter '+IntToStr(x)
    else
      LabelText:=' '; // we must have at least on space here (Big in TEE-Chart?)
  end;
end;

procedure TForm1.prepareBoxPlot(Ymin,YMax:real;Count:integer);
var r:trect;
begin
  boxPlotActive:=true;
  BoxCount:=count;
  // Set ranges for x- and y-axis
  with chart1.leftaxis do begin
    automatic:=true;
    automaticMinimum:=false;
    automaticMaximum:=false;
    maximum:=1E20;
    minimum:=YMin;
    maximum:=YMax;
  end;
  with chart1.bottomaxis do begin
    automatic:=true;
    automaticMinimum:=false;
    automaticMaximum:=false;
    minimum:=0;
    maximum:=count+1;
  end;
  // Calculate misc variables
  // First let TChart recalculate the axes
  application.processMessages; // important!!
  // Get effective area of chart
  r:=chart1.chartRect;
  bDX:=(r.right-r.left) / (count+1);
  // Since Windows uses y direction bottom up, we make this negative
  bDY:=-(r.bottom-r.top) / (YMax-YMin);
  bX0:=r.left;
  bY0:=r.top;
  byy0:=Ymax;
end;

function TForm1.PixY(y:real):integer;
// Calculate Pixelvalue from Y-Value
begin
  result:=round(by0-(byy0-y)*bdy)
end;

procedure TForm1.freeBoxPlot;
begin
  boxPlotActive:=false;
  with chart1.leftaxis do begin
    automatic:=true;
    automaticMinimum:=true;
    automaticMaximum:=true;
  end;
  with chart1.bottomaxis do begin
    automatic:=true;
    automaticMinimum:=true;
    automaticMaximum:=true;
  end;
  series1.clear;
end;

procedure TForm1.DrawBox(x:integer;y0,y1,y2,y3,y4,ym:real);
var OldWidth:integer;
begin
  with chart1.canvas do begin
    OldWidth:=pen.Width;
    pen.Width:=2;
    // Body Rectangle (y1 to y3)
    rectangle(round(bx0+x*bdx-bwb div 2-dist),pixy(y1),
              round(bx0+(x)*bdx+bwb div 2-dist),pixY(y3));
    // Median (y2)
    moveto(round(bx0+x*bdx-bwb div 2-dist+1),pixy(y2));
    lineto(round(bx0+x*bdx+bwb div 2-dist -1),pixy(y2));
    // Mean (ym)
    moveto(round(bx0+x*bdx-dist),pixy(ym)-bWM);
    lineto(round(bx0+x*bdx-dist),pixy(ym)+bWM);
    moveto(round(bx0+x*bdx-dist)-bwm,pixy(ym));
    lineto(round(bx0+x*bdx-dist)+bwm,pixy(ym));
    // Antenna y0
    moveto(round(bx0+x*bdx-dist),pixy(y0));
    lineto(round(bx0+x*bdx-dist),pixy(y1));
    moveto(round(bx0+x*bdx-dist-bWA),pixy(y0));
    lineto(round(bx0+x*bdx-dist)+bWA,pixy(y0));
    // Antenna y4
    moveto(round(bx0+x*bdx-dist),pixy(y3));
    lineto(round(bx0+x*bdx-dist),pixy(y4));
    moveto(round(bx0+x*bdx-dist-bWA),pixy(y4));
    lineto(round(bx0+x*bdx-dist)+bWA,pixy(y4));
    pen.Width:=OldWidth;
  end;
end;

procedure TForm1.DrawPoint(x:integer;y:real);
// Individual value to be shown beside the boxes
var oldColor:TColor;
begin
  with chart1.canvas do begin
    oldColor:=brush.color;
    brush.color:=clBlack;
    fillRect(rect(round(bx0+x*bdx-bwm)+dist,pixy(y)-bwm,
             round(bx0+x*bdx+bwm)+dist,pixY(y)+bwm));
    brush.color:=oldColor;;
  end;
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  freeBoxPlot;
end;


procedure TForm1.Button3Click(Sender: TObject);
begin
  // You will have to redraw this every time the chart is resized!
  PrepareBoxPlot(4.5,16.6,5);

  DrawBox(1,5.2,6.1,7.5,9.2,15.5,8.1);
  DrawPoint(1,8.45);

  DrawBox(2,5.1,6.4,7.8,10.2,14.5,7.1);
  DrawPoint(2,8.45);

  DrawBox(3,6.2,7.1,7.9,11.44,16.1,8.3);
  DrawPoint(3,8.45);

  DrawBox(4,5.2,6.1,7.5,9.2,15.5,8.1);
  DrawPoint(4,8.45);

  DrawBox(5,5.1,6.4,7.8,10.2,14.5,7.1);
  DrawPoint(5,8.45);
end;

end.

